/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the yifu4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */

package com.yifu.cloud.plus.v1.yifu.archives.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yifu.cloud.plus.v1.yifu.archives.entity.TCutsomerDataPermisson;
import com.yifu.cloud.plus.v1.yifu.archives.service.TCutsomerDataPermissonService;
import com.yifu.cloud.plus.v1.yifu.common.core.util.R;
import com.yifu.cloud.plus.v1.yifu.common.log.annotation.SysLog;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;


/**
 * 客户数据权限表
 *
 * @author hgw
 * @date 2022-06-20 09:59:17
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/tcutsomerdatapermisson")
@Tag(name = "客户数据权限表管理")
public class TCutsomerDataPermissonController {

	private final TCutsomerDataPermissonService tCutsomerDataPermissonService;

	/**
	 * 分页查询
	 *
	 * @param page                   分页对象
	 * @param tCutsomerDataPermisson 客户数据权限表
	 * @return
	 */
	@Operation(summary = "分页查询", description = "分页查询")
	@GetMapping("/page")
	@PreAuthorize("@pms.hasPermission('demo_tcutsomerdatapermisson_get')")
	public R<IPage<TCutsomerDataPermisson>> getTCutsomerDataPermissonPage(Page<TCutsomerDataPermisson> page, TCutsomerDataPermisson tCutsomerDataPermisson) {
		return R.ok(tCutsomerDataPermissonService.page(page, Wrappers.query(tCutsomerDataPermisson)));
	}


	/**
	 * 通过id查询客户数据权限表
	 *
	 * @param id id
	 * @return R
	 */
	@Operation(summary = "通过id查询", description = "通过id查询")
	@GetMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('demo_tcutsomerdatapermisson_get')")
	public R<TCutsomerDataPermisson> getById(@PathVariable("id") String id) {
		return R.ok(tCutsomerDataPermissonService.getById(id));
	}

	/**
	 * 新增客户数据权限表
	 *
	 * @param tCutsomerDataPermisson 客户数据权限表
	 * @return R
	 */
	@Operation(summary = "新增客户数据权限表", description = "新增客户数据权限表")
	@SysLog("新增客户数据权限表")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('demo_tcutsomerdatapermisson_add')")
	public R<Boolean> save(@RequestBody TCutsomerDataPermisson tCutsomerDataPermisson) {
		return R.ok(tCutsomerDataPermissonService.save(tCutsomerDataPermisson));
	}

	/**
	 * 修改客户数据权限表
	 *
	 * @param tCutsomerDataPermisson 客户数据权限表
	 * @return R
	 */
	@Operation(summary = "修改客户数据权限表", description = "修改客户数据权限表")
	@SysLog("修改客户数据权限表")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('demo_tcutsomerdatapermisson_edit')")
	public R<Boolean> updateById(@RequestBody TCutsomerDataPermisson tCutsomerDataPermisson) {
		return R.ok(tCutsomerDataPermissonService.updateById(tCutsomerDataPermisson));
	}

	/**
	 * 通过id删除客户数据权限表
	 *
	 * @param id id
	 * @return R
	 */
	@Operation(summary = "通过id删除客户数据权限表", description = "通过id删除客户数据权限表")
	@SysLog("通过id删除客户数据权限表")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('demo_tcutsomerdatapermisson_del')")
	public R<Boolean> removeById(@PathVariable String id) {
		return R.ok(tCutsomerDataPermissonService.removeById(id));
	}

}
